/*  $Id: StrucFindDialog.cpp,v 1.29 2003/05/22 16:43:09 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <qaccel.h>
#include <qpushbutton.h>

#include "StrucFindDialog.h"

#define Inherited StrucFindDialogData

#include "StrucFindDialog.moc"

StrucFindDialog::StrucFindDialog(const QString& title, QWidget* parent,const char* name) :
  Inherited( parent, name )
{
  accel_CtrlG = new QAccel(this);
  accel_CtrlG->connectItem(accel_CtrlG->insertItem(CTRL + Key_G), this,
  			   SLOT(nextSlot()));
  accel_CtrlG->setEnabled(FALSE);

  setCaption(title);
  input->setFocus();
  ignoreCase->setChecked(true);
  qtarch_PushButton_3->setEnabled(FALSE);

  positionDialog();
}

StrucFindDialog::StrucFindDialog(const QString& title, char* searchTerm, QWidget* parent,const char* name) :
  Inherited( parent, name )
{
  accel_CtrlG = new QAccel(this);
  accel_CtrlG->connectItem(accel_CtrlG->insertItem(CTRL + Key_G), this,
			   SLOT(nextSlot()));
  
  setCaption(title);
  input->setFocus();
  input->setText(QString::fromLocal8Bit(searchTerm));
  qtarch_PushButton_3->setDefault(TRUE);
  qtarch_PushButton_3->setEnabled(TRUE);

  ignoreCase->setChecked(false);

  accel_CtrlG->setEnabled(TRUE);

  positionDialog();
}


StrucFindDialog::~StrucFindDialog()
{
}


void StrucFindDialog::positionDialog(void) {
  // Geometry of the view window
  int px = parentWidget()->x();
  int py = parentWidget()->y();
  int pw = parentWidget()->width();

  // Width of the desktop
  int sw = QApplication::desktop()->width();

  // The width of this dialog
  int w = width();

  // The new top left coordinates of this dialog
  int x;
  int y;

  // Enough space to the right of the view window?
  // Position the dialog at the top right
  if (px + pw < sw - w - 10) {
    x = px + pw + 10;
    y = py;
  }
  // Enough space to the left of the view window?
  // Position the dialog at the top left
  else if (px > w + 10) {
    x = px - w - 10;
    y = py;
  }
  // No space to either left or right (view window is too big)
  // Position the dialog on the far right of the desktop, on the
  // same height as the view
  else {
    x = sw - w;
    y = py;
  }
  
  setGeometry(x, y, width(), height());

  if (parentWidget())
    setIcon(*(parentWidget()->icon()));
}


void StrucFindDialog::okSlot() {
  emit findSignal();
  qtarch_PushButton_3->setDefault(TRUE);
  qtarch_PushButton_3->setEnabled(TRUE);
  accel_CtrlG->setEnabled(TRUE);
  
}

void StrucFindDialog::cancelSlot() {
  close();
}

void StrucFindDialog::nextSlot() {
  emit findNextSignal();
  qtarch_PushButton_3->setDefault(TRUE);
}


void StrucFindDialog::textChangedSlot(const QString& text) {
  qtarch_PushButton_1->setDefault(TRUE);
  qtarch_PushButton_3->setEnabled(FALSE);
  accel_CtrlG->setEnabled(FALSE);
}


void StrucFindDialog::ignoreCaseToggledSlot(bool on) {
  qtarch_PushButton_1->setDefault(TRUE);
  qtarch_PushButton_3->setEnabled(FALSE);
  accel_CtrlG->setEnabled(FALSE);
}  


const QString StrucFindDialog::text() {
  return input->text();
}


bool StrucFindDialog::ignoreCaseChecked(void) {
  return ignoreCase->isChecked();
}


void StrucFindDialog::closeEvent(QCloseEvent* ev) {
  emit cancelSignal();
  ev->accept();
}

void StrucFindDialog::reject() {
  cancelSlot();
}
