/*  $Id: compare.h,v 1.29 2003/05/22 16:43:09 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file

    Grisu text-based structure compare algorithm.

    WARNING!!! Since Grisu is mainly used as the frontend for the TRALE system, which has
    built-in structure comparison functionality, this algorithm has NOT BEEN extended
    to support newer node types like NODETYPE_DISJUNCTION_NODE. If you want to use this
    code, you MUST add the appropriate handling of these node types.

    @author Holger Wunsch */
#ifndef __COMPARE_H
#define __COMPARE_H

#include "data.h"
#include "node.h"
#include "NodeAddress.h"
#include "idtree.h"

class StructureCompare {
  Document* document1;
  const NodeAddress& ad1;
  GenericNode* nd1;
  IdTree* tree1;
  Document* document2;
  const NodeAddress& ad2;
  GenericNode* nd2;
  IdTree* tree2;
  bool cyclic;
  int diffs;

 public:

  StructureCompare(Document* document1, const NodeAddress& addr1, GenericNode* node1, IdTree* tree1, Document* document2, const NodeAddress& addr2, GenericNode* node2, IdTree* top2);

  ~StructureCompare();

  void compare();

  bool cyclicPaths();

  int differences();

 private:
  void markAsDifferent(const NodeAddress& addr, GenericNode* node, IdTree* tree);

  void compareGenericNodes(const NodeAddress& addr1, GenericNode* node1, const NodeAddress& addr2, GenericNode* node2);
  void compareTypeNodes(const NodeAddress& parentAddr1, TypeNode* node1, const NodeAddress& parentAddr2, TypeNode* node2);
  void compareFeatvalNodes(const NodeAddress& parentAddr1, FeatvalNode* node1, const NodeAddress& parentAddr2, FeatvalNode* node2);
  void compareStrucNodes(const NodeAddress& parentAddr1, StrucNode* node1, const NodeAddress& parentAddr2, StrucNode* node2);
  void compareTailNodes(const NodeAddress& parentAddr1, TailNode* node1, const NodeAddress& parentAddr2, TailNode* node2);
  void compareListNodes(const NodeAddress& parentAddr1, ListNode* node1, const NodeAddress& parentAddr2, ListNode* node2);
  void compareSetNodes(const NodeAddress& parentAddr1, SetNode* node1, const NodeAddress& parentAddr2, SetNode* node2);
  void compareRestNodes(const NodeAddress& parentAddr1, RestNode* node1, const NodeAddress& parentAddr2, RestNode* node2);
  void compareFunctionNodes(const NodeAddress& parentAddr1, FunctionNode* node1, const NodeAddress& parentAddr2, FunctionNode* node2);
  void compareRelationNodes(const NodeAddress& parentAddr1, RelationNode* node1, const NodeAddress& parentAddr2, RelationNode* node2);
  void compareRefNodes(const NodeAddress& parentAddr1, RefNode* node1, const NodeAddress& parentAddr2, RefNode* node2);
  void compareReentNodes(const NodeAddress& parentAddr1, ReentNode* node1, const NodeAddress& parentAddr2, ReentNode* node2);
  void compareTreeNodes(const NodeAddress& parentAddr1, TreeNode* node1, const NodeAddress& parentAddr2, TreeNode* node2);
  void compareAtomNodes(const NodeAddress& parentAddr1, AtomNode* node1, const NodeAddress& parentAddr2, AtomNode* node2);
};


#endif
