/*  $Id: dispnode.h,v 1.29 2003/05/22 16:43:11 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file
    Declaration of the DisplayableNode class.
    @author Holger Wunsch */

#ifndef __DISPNODE_H
#define __DISPNODE_H

#include "node.h"
#include "NodeAddress.h"


/** Instances of DisplayableNode contain information about a node that is visible
    in a view window. Displayable nodes are most important in the context of reentrant structures.
    In a Document, a reentrant structure is represented only once, but on-screen, the same structure
    may appear in several different places. Changes, applied by the user to the appearance of one node, should
    not affect the appearance of other nodes, even if they are the same shared structure. Therefore, in addition
    to the Node based data structure, for each node shown on the screen, a DisplayableNode is created that contains
    all data specific to one individual displayed node.
    @author Holger Wunsch */
struct DisplayableNode {
  /** The unique node address of the displayed node. */
  NodeAddress addr;

  
  /** A pointer to the node data. */
  GenericNode* node;


  /** The flags field overrides the flags field in node. When a node is drawn, the values in this flags field
      determine the visual appearance of the node. */
  unsigned int flags;// Flags as defined in node.h
};

#endif
