/*  $Id: il.y,v 1.36 2003/05/22 16:43:12 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


%pure_parser


%{
  #include <stdio.h>
  #include <stdlib.h>
  #include <kmessagebox.h>
  #include "node.h"

  #include "dataparser.h"
  

#define  YYERROR_VERBOSE 1
#define  YYDEBUG 1
#define  YYPARSE_PARAM arg
#define  YYLEX_PARAM arg

 int yyerror(char* s); 

 extern int curLoc;

 /* Twice the default stack size to avoid stack overflow errors when parsing large data packages. */
#define YYMAXDEPTH 20000

#define YYINITDEPTH 200

%}

%union {
  char* string_type;
  int int_type;
  GenericNode* generic_node_type;
  TypeNode* type_node_type;
  StrucNode* struc_node_type;
  FeatvalNode* featval_node_type;
  ListNode* list_node_type;
  SetNode* set_node_type;
  FunctionNode* function_node_type;
  RelationNode* rel_node_type;
  RefNode* ref_node_type;
  ReentNode* reent_node_type;
  TreeNode* tree_node_type;
  AtomNode* atom_node_type;
  TailNode* tail_node_type;
  RestNode* rest_node_type;
  DisjunctionNode* disjunction_node_type;
  ConjunctionNode* conjunction_node_type;
  List* list_type;
}

%{
  int yylex(YYSTYPE*, void*);
  %}

%token <string_type> STRING
%token <string_type> ATOM
%token <int_type> INTEGER
%token NEWDATA
%token NEWDATA_TREE
%token NEWDATA_STRUCTURE
%token STRUC_MARKER
%token TREE_MARKER
%token REENT_MARKER
%token FEATVAL_MARKER
%token LIST_MARKER
%token SET_MARKER
%token FUNCTION_MARKER
%token REL_MARKER
%token TREE_MARKER
%token ATOM_MARKER
%token TAIL_MARKER
%token REST_MARKER
%token DISJUNCTION_MARKER
%token CONJUNCTION_MARKER

%token LEFT_PAREN
%token RIGHT_PAREN

%token SLASH

%token HASH

%token HIDDEN_FLAG
%token DIFFERENT_FLAG
%token STRIKEOUT_FLAG
%token MULTILINE_FLAG
%token EXPANDED_FLAG

%token ERROR

%type <int_type> newdata
%type <list_type> datapackage
%type <list_type> structures
%type <generic_node_type> structure
%type <list_type> structs
%type <generic_node_type> struct
%type <list_type> trees
%type <tree_node_type> tree
%type <struc_node_type> struc
%type <list_type> featvals
%type <featval_node_type> featval
%type <disjunction_node_type> disjunction
%type <conjunction_node_type> conjunction
%type <list_node_type> list
%type <tail_node_type> tail
%type <set_node_type> set
%type <rest_node_type> rest
%type <function_node_type> function
%type <rel_node_type> relation
%type <type_node_type> type
%type <string_type> feature
%type <string_type> functor
%type <ref_node_type> ref
%type <atom_node_type> any
%type <reent_node_type> reentr
%type <string_type> windowtitle
%type <int_type> flags
%type <int_type> flag
%type <string_type> label
%type <string_type> arclabel
%type <int_type> id
%type <int_type> linkid

%start datapackage

%% 

datapackage:	newdata windowtitle structures {$$ = $3; ((ParserArgs*) arg)->parsedData = $$; strcpy(((ParserArgs*) arg)->windowtitle, $2); ((ParserArgs*) arg)->showDefault = $1;};

newdata:        NEWDATA {$$ = 0;} |
                NEWDATA_TREE {$$ = DRAW_TREE;} |
                NEWDATA_STRUCTURE {$$ = DRAW_STRUC;};

structures:	structure {$$ = prepend($1, NULL);}|
		structures structure {$$ = append($1, $2);};
structure:	struct {$$ = (GenericNode*) $1;} |
		reentr {$$ = (GenericNode*) $1;} | 
		tree {$$ = (GenericNode*) $1;};

structs:	{$$ = NULL;} | structs struct {$$ = append($1, (GenericNode*) $2);};
struct:		struc {$$ = (GenericNode*) $1;} | 
		ref {$$ = (GenericNode*) $1;} | 
		function {$$ = (GenericNode*) $1;} |
		relation {$$ = (GenericNode*) $1;} |
		list {$$ = (GenericNode*) $1;} | 
		set {$$ = (GenericNode*) $1;} | 
		any {$$ = (GenericNode*) $1;} |
		disjunction {$$ = $1; } |
		conjunction {$$ = $1; };

trees:		{$$ = NULL;} | 
		trees tree {$$ = append($1, (GenericNode*) $2);};
tree:		LEFT_PAREN TREE_MARKER flags id label arclabel linkid trees RIGHT_PAREN {$$ = createTree($3, $4, $5, $6, $7, $8); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

reentr:		LEFT_PAREN REENT_MARKER flags id INTEGER struct RIGHT_PAREN {$$ = createReent($3, $4, $5, $6); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

struc:		LEFT_PAREN STRUC_MARKER flags id type featvals RIGHT_PAREN {$$ = createStruc($3, $4, $5, $6); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

featvals:	{$$ = NULL;}| featval featvals {$$ = prepend((GenericNode*) $1, $2);};
featval:	LEFT_PAREN FEATVAL_MARKER flags id feature struct RIGHT_PAREN {$$ = createFeatval($3, $4, $5, $6);((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);} |
                LEFT_PAREN FEATVAL_MARKER flags id feature RIGHT_PAREN {$$ = createFeatval($3, $4, $5, NULL); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

disjunction:    LEFT_PAREN DISJUNCTION_MARKER flags id struct struct structs RIGHT_PAREN {$$ = createDisjunction($3, $4, prepend($5, prepend($6, $7))); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);}; 

conjunction:    LEFT_PAREN CONJUNCTION_MARKER flags id struct struct structs RIGHT_PAREN {$$ = createConjunction($3, $4, prepend($5, prepend($6, $7))); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);}; 

list:		LEFT_PAREN LIST_MARKER flags id structs tail RIGHT_PAREN {$$ = createList($3, $4, $5, $6); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

tail:		{$$ = NULL;} | LEFT_PAREN TAIL_MARKER flags id struct RIGHT_PAREN {$$ = createTail($3, $4, $5); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

set:		LEFT_PAREN SET_MARKER flags id structs rest RIGHT_PAREN {$$ = createSet($3, $4, $5, $6); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

rest:		{$$ = NULL;} | LEFT_PAREN REST_MARKER flags id struct RIGHT_PAREN {$$ = createRest($3, $4, $5); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

function:	LEFT_PAREN FUNCTION_MARKER flags id type functor structs RIGHT_PAREN {$$ = createFunction($3, $4, $5, $6, $7); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);} ;

relation:	LEFT_PAREN REL_MARKER flags id functor structs RIGHT_PAREN {$$ = createRelation($3, $4, $5, $6); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);} ;

type:		LEFT_PAREN flags id STRING RIGHT_PAREN {$$ = createType($2, $3, $4); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $3);};

ref:		LEFT_PAREN HASH flags id INTEGER RIGHT_PAREN {$$ = createRef($3, $4, $5); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

any:		LEFT_PAREN ATOM_MARKER flags id STRING RIGHT_PAREN {$$ = createAtom($3, $4, $5); ((ParserArgs*) arg)->map = addToMap(((ParserArgs*) arg)->map, (GenericNode*) $$, $4);};

feature:	STRING {$$ = $1;};

functor:	STRING {$$ = $1;};

windowtitle:	STRING;

label:		STRING;

arclabel:       {$$ = NULL;} |
                STRING {$$ = $1;};

flags:          flag flags {$$ = $1 | $2;} | 
                {$$ = 0;};

flag:           HIDDEN_FLAG {$$ = NODEFLAG_HIDDEN;} |
                DIFFERENT_FLAG {$$ = NODEFLAG_DIFFERENT;} |
		STRIKEOUT_FLAG {$$ = NODEFLAG_STRIKEOUT;} |
		MULTILINE_FLAG {$$ = NODEFLAG_MULTILINE;} |
                EXPANDED_FLAG {$$ = NODEFLAG_EXPANDED;};

id:		INTEGER;

linkid:		INTEGER;



		
%%

int yyerror(char *s) {
  char buf[500];
  sprintf(buf, "The input data is corrupt.\nThe error message generated by the parser is:\n%s\nPosition: %d", s, curLoc);
  KMessageBox::error(NULL, buf, "Parse Error", false);
  return 0;
}

