/*  $Id: main.cpp,v 1.36 2003/05/24 01:55:02 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @file

   Grisu main program file.

   @author Holger Wunsch */

/** @mainpage

    <center>
    <h2>GRammar vISUalization tool (GRISU) - Technical Documentation</h2>
    <h3>Version 1.3.x</h3>

    Developed by Holger Wunsch and Detmar Meurers<br>
    Copyright (C) 1998-2003 by the developers<br>
    All rights reserved<br>

    Seminar fr Sprachwissenschaft, Universitt Tbingen and<br>
    Department of Linguistics, The Ohio State University


    For a documentation of the Grisu interface language syntax, see the
    <a href="../interface-language-ebnf/ebnf.txt">interface-language EBNF</a>
    </center>
*/


#include <qfiledlg.h>
#include <qtextcodec.h>

#include <string.h>
#include <stdlib.h>
#include <kapp.h>

#include <qstringlist.h>

#include <kmessagebox.h>

#include <kstddirs.h>
#include <kcmdlineargs.h>

#include "mainw.h"
#include "data.h" 
#include "options.h"

#include "config.h"

#include "gdataexchange.h"


/** A list of loaded documents. */
QList<Document> documents;


/** The global application options. */
Options *globalOptions;


/** <code>true</code> if an options was changed during a Grisu session, <code>false</code> otherwise. */
bool optionChanged;


/** The main window. */
MainWindow *mainw;


/** The application object */
KApplication *application;


/** The application-wide data exchange manager. */
GDataExchange* dataxc;


/** If this variable is set to <code>true</code>, Grisu will dump all
    data coming in on its socket connection to the file <code>grisu.dmp</code>
    in the directory where Grisu was started. */
bool dumpData;


/** Hardcoded resource path. This can be set to a resource path in the makefile during compilation.
    This value will be used if the <code>GRISU_RESOURCE_PATH</code> environment variable is not set
    on startup. */
#ifndef RESOURCE_PATH
#define RESOURCE_PATH ""
#endif


/** The resource path, which is the path where Grisu resources (icons, for example) are stored. */
char* resourcePath;


/** Program entry point. */
int main( int argc, char *argv[] )
{
  char* resPath = getenv("GRISU_RESOURCE_PATH");
  if (resPath)
    resourcePath = strdup(resPath);
  else
    resourcePath = strdup(RESOURCE_PATH);

  KCmdLineArgs::init(argc, argv, APP_NAME, "", GRISU_VERSION);

  application = new KApplication();

  KApplication::kApplication()->dirs()->addResourceDir("icon", resourcePath);
  KApplication::kApplication()->newIconLoader();

  globalOptions = new Options;
  globalOptions->writeOptionFromConfig();

  mainw         = new MainWindow;
  mainw->show();

  dataxc = new GDataExchange(mainw);
  QObject::connect(dataxc, SIGNAL(connectionStateSignal(unsigned char, char*, int)),
		   mainw, SLOT(connectionStateSlot(unsigned char, char*, int)));
  dataxc->setupServerSocket();

  // Parse command line 
  if (argc > 1) {
    for (int i = 1; i < argc; i++) {
      if (strcmp(argv[i], "-dump") == 0)
	dumpData = true;
      else
	mainw->loadDataFile(argv[i]);
    }
  }
  
  optionChanged = false;

  // the main loop
  int ret = application->exec();

  if (optionChanged) {
    if (KMessageBox::questionYesNo(NULL, "You have changed several options during this Grisu session.\n\n"
				   "Would you like to make the new settings permanent?",
				   "Modified Options") == KMessageBox::Yes)
      globalOptions->writeOptionFromConfig();
  }
  // Never forget to remove window-structures from X-Windows
  // ... but NOT with Qt 2, where widgets are deleted automatically...
  //delete mainw;

  delete dataxc;

  delete globalOptions;
  delete application;

  free(resourcePath);
  
  return ret;
}


