/*  $Id: node.cpp,v 1.35 2003/05/22 16:43:13 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdlib.h>
#include <string.h>
#include "node.h"
#include "dispnode.h"
#include "NodeAddress.h"
#include "idtree.h"
#include "canvas.h"


void GenericNode::hideUnhideChildNodes(Canvas* canvas, NodeAddress& na, bool hide) {
  DisplayableNode* nd = canvas->getDispNodes()->findDisplayableNode(na);
  if (nd)
    fprintf(stderr, "GenericNode::unhideChildNodes: Unknown node type %d\n", nd->node->nodetype);
}


void GenericNode::unhideChildNode(Canvas* canvas, NodeAddress& addr, int index) {
  fprintf(stderr, "unhideChildNode() called with unsupported node type\n");
}


void StrucNode::hideUnhideChildNodes(Canvas* canvas, NodeAddress& na, bool hide) {
  NodeAddress addrChild;
  DisplayableNode* nd;
  IdTree* dispNodes = canvas->getDispNodes();

  addrChild = NodeAddress(na, type);
  nd = dispNodes->addDisplayableNode(addrChild, type);
  if (nd) {
    if (hide)
      nd->flags &= ~NODEFLAG_SHOWHIDDEN;
    else
      nd->flags |= NODEFLAG_SHOWHIDDEN;
  }

  List* fv = featvals;
  while (fv) {
    addrChild = NodeAddress(na, fv->item);
    nd = dispNodes->addDisplayableNode(addrChild, fv->item);
    if (nd) {
      if (hide)
	nd->flags &= ~NODEFLAG_SHOWHIDDEN;
      else
	nd->flags |= NODEFLAG_SHOWHIDDEN;
    }
    fv = fv->tail;
  }
}


void StrucNode::unhideChildNode(Canvas* canvas, NodeAddress& addr, int index) {
  DisplayableNode* dnd;
  IdTree* dispNodes = canvas->getDispNodes();
  int ndIndex = 0;

  dnd = dispNodes->addDisplayableNode(NodeAddress(addr, type), type);
  if ((dnd->flags & NODEFLAG_HIDDEN) && !(dnd->flags & NODEFLAG_SHOWHIDDEN)) {
    if (ndIndex == index) {
      //dnd->flags |= NODEFLAG_SHOWHIDDEN;
      dnd->flags &= ~NODEFLAG_HIDDEN;
      return;
    }
    else
      ndIndex++;
  }

  List* fv = featvals;
  while (fv) {
    dnd = dispNodes->addDisplayableNode(NodeAddress(addr, fv->item), fv->item);
    if ((dnd->flags & NODEFLAG_HIDDEN) && !(dnd->flags & NODEFLAG_SHOWHIDDEN)) {
      if (ndIndex == index) {
	//dnd->flags |= NODEFLAG_SHOWHIDDEN;
	dnd->flags &= ~NODEFLAG_HIDDEN;
	break;
      }
      else
	ndIndex++;
    }
    fv = fv->tail;
  }
}


TypeNode* createType(unsigned int flags, int id, char* name)
{
  TypeNode* node = new TypeNode;

  node->nodetype = NODETYPE_TYPE_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->name = strdup(name);

  return node;
};

void destroyType(TypeNode* node)
{
  delete [] node->name;

  delete node;
}

StrucNode* createStruc(unsigned int flags, int id, TypeNode* type, List* featvals)
{
  StrucNode* node = new StrucNode;

  node->nodetype = NODETYPE_STRUC_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->type = type;
  node->featvals = featvals;

  return node;
}

void destroyStruc(StrucNode* node)
{
  freeList(node->featvals);

  delete node;
}

FunctionNode* createFunction(unsigned int flags, int id, TypeNode* type, char* functor, List* structs)
{
  FunctionNode* node = new FunctionNode;

  node->nodetype = NODETYPE_FUNCTION_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->type = type;
  node->functor = strdup(functor);
  node->structs = structs;

  return node;
};

void destroyFunction(FunctionNode* node)
{
  delete [] node->functor;
  freeList(node->structs);

  delete node;
}

RelationNode* createRelation(unsigned int flags, int id, char* functor, List* structs)
{
  RelationNode* node = new RelationNode;

  node->nodetype = NODETYPE_REL_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->functor = strdup(functor);
  node->structs = structs;

  return node;
};

void destroyRelation(RelationNode* node)
{
  delete [] node->functor;
  freeList(node->structs);

  delete node;
}

SetNode* createSet(unsigned int flags, int id, List* structs, RestNode* rest)
{
  SetNode* node = new SetNode;

  node->nodetype = NODETYPE_SET_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->structs = structs;
  node->rest = rest;

  return node;
};

void destroySet(SetNode* node)
{
  freeList(node->structs);

  delete node;
}

ListNode* createList(unsigned int flags, int id, List* structs, TailNode* tail)
{
  ListNode* node = new ListNode;

  node->nodetype = NODETYPE_LIST_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->structs = structs;
  node->tail = tail;

  return node;
};

void destroyList(ListNode* node)
{
  freeList(node->structs);
 
  delete node;
}

FeatvalNode* createFeatval(unsigned int flags, int id, char* feature, GenericNode* value)
{
  FeatvalNode* node = new FeatvalNode;

  node->nodetype = NODETYPE_FEATVAL_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->feature = strdup(feature);
  node->value = value;

  return node;
};

void destroyFeatval(FeatvalNode* node)
{
  delete [] node->feature;

  delete node;
}

RefNode* createRef(unsigned int flags, int id, int ref) {
  RefNode* node = new RefNode;

  node->nodetype = NODETYPE_REF_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;
  
  node->ref = ref;

  return node;
}

void destroyRef(RefNode* node) {
  delete node;
}

ReentNode* createReent(unsigned int flags, int id, int ref, GenericNode* value) {
  ReentNode* node = new ReentNode;

  node->nodetype = NODETYPE_REENT_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->ref = ref;
  node->value = value;

  return node;
}

void destroyReent(ReentNode* node) {
  delete node;
}

TreeNode* createTree(unsigned int flags, int id, char* label, char* arclabel, int linkid, List* subtrees) {
  TreeNode* node = new TreeNode;

  node->nodetype = NODETYPE_TREE_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;
  node->label = strdup(label);
  if (arclabel != NULL)
    node->arclabel = strdup(arclabel);
  else
    node->arclabel = NULL;
  sprintf(node->linkid, "%d", linkid);
  node->subtrees = subtrees;

  return node;
}

void destroyTree(TreeNode* node) {
  delete [] node->label;
  if (node->arclabel != NULL)
    delete [] node->arclabel;
  freeList(node->subtrees);
  delete node;
}

AtomNode* createAtom(unsigned int flags, int id, char* atom) {
  AtomNode* node = new AtomNode;

  node->nodetype = NODETYPE_ATOM_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->atom = strdup(atom);

  return node;
}

void destroyAtom(AtomNode* node) {
  delete [] node->atom;
  delete node;
}

TailNode* createTail(unsigned int flags, int id, GenericNode* tail) {
  TailNode* node = new TailNode;

  node->nodetype = NODETYPE_TAIL_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->tail = tail;
  
  return node;
}

void destroyTail(TailNode* node) {
  delete node;
}

RestNode* createRest(unsigned int flags, int id, GenericNode* rest) {
  RestNode* node = new RestNode;

  node->nodetype = NODETYPE_REST_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->rest = rest;
  
  return node;
}

void destroyRest(RestNode* node) {
  delete node;
}


DisjunctionNode* createDisjunction(unsigned int flags, int id, List* operands) {
  DisjunctionNode* node = new DisjunctionNode;

  node->nodetype = NODETYPE_DISJUNCTION_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->operands = operands;

  return node;
}


void destroyDisjunction(DisjunctionNode* node) {
  freeList(node->operands);

  delete node;
}


ConjunctionNode* createConjunction(unsigned int flags, int id, List* operands) {
  ConjunctionNode* node = new ConjunctionNode;

  node->nodetype = NODETYPE_CONJUNCTION_NODE;
  sprintf(node->id, "%d", id);
  node->flags = flags;

  node->operands = operands;

  return node;
}


void destroyConjunction(ConjunctionNode* node) {
  freeList(node->operands);

  delete node;
}


void destroyNode(GenericNode* node)
{
  if (node == NULL)
    return;
  switch (node->nodetype) {
  case NODETYPE_TYPE_NODE:
    destroyType((TypeNode*) node);
    break;
  case NODETYPE_STRUC_NODE:
    destroyStruc((StrucNode*) node);
    break;
  case NODETYPE_FEATVAL_NODE:
    destroyFeatval((FeatvalNode*) node);
    break;
  case NODETYPE_LIST_NODE:
    destroyList((ListNode*) node);
    break;
  case NODETYPE_SET_NODE:
    destroySet((SetNode*) node);
    break;
  case NODETYPE_FUNCTION_NODE:
    destroyFunction((FunctionNode*) node);
    break;
  case NODETYPE_REL_NODE:
    destroyRelation((RelationNode*) node);
    break;
  case NODETYPE_REF_NODE:
    destroyRef((RefNode*) node);
    break;
  case NODETYPE_REENT_NODE:
    destroyReent((ReentNode*) node);
    break;
  case NODETYPE_TREE_NODE:
    destroyTree((TreeNode*) node);
    break;
  case NODETYPE_ATOM_NODE:
    destroyAtom((AtomNode*) node);
    break;
  case NODETYPE_TAIL_NODE:
    destroyTail((TailNode*) node);
    break;
  case NODETYPE_REST_NODE:
    destroyRest((RestNode*) node);
    break;
  case NODETYPE_DISJUNCTION_NODE:
    destroyDisjunction((DisjunctionNode*) node);
    break;
  case NODETYPE_CONJUNCTION_NODE:
    destroyConjunction((ConjunctionNode*) node);
    break;
  default:
    printf("Illegal node type %d\n", node->nodetype);
    break;
  }
}


List* prepend(GenericNode* item, List* tail)
{
  List* list = new List;

  list->item = item;
  list->tail = tail;

  return list;
};

List* append(List* l, GenericNode* item)
{
  List* p = l;
  List* list = new List;
  list->item = item;
  list->tail = NULL;
  
  if (!l)
    return list;

  while (p->tail)
    p = p->tail;

  p->tail = list;

  return l;
}
    
void freeList(List* list)
{
  List* del;

  while (list) {
    del = list;

    list = del->tail;
    delete del;
  }
}


void printNode(GenericNode* node, int level);

void catList(List* list, int level)
{
  while (list) {
    printNode(list->item, level);
    list = list->tail;
  }
}

void space(int level)
{
  while (level > 0) {
    printf(" ");
    level--;
  }
}

void writeFlags(FILE* file, GenericNode* node) {
  if (node->flags & NODEFLAG_HIDDEN)
    fprintf(file, "+");
  if (node->flags & NODEFLAG_DIFFERENT)
    fprintf(file, "*");
  if (node->flags & NODEFLAG_STRIKEOUT)
    fprintf(file, "-");
  if (node->flags & NODEFLAG_MULTILINE)
    fprintf(file, "|");
  if (node->flags & NODEFLAG_EXPANDED)
    fprintf(file, ">");
}
  
void writeTypeNode(FILE* file, GenericNode* node) {
  TypeNode* typenode = (TypeNode*) node;
  
  fprintf(file, "(");
  writeFlags(file, node);
  fprintf(file, "%s", typenode->id);
  fprintf(file, "\"%s\"", typenode->name);
  fprintf(file, ")");
}

void writeStrucNode(FILE* file, GenericNode* node) {
  StrucNode* strucnode = (StrucNode*) node;
  List* featvals;

  fprintf(file, "(S");
  writeFlags(file, node);
  fprintf(file, "%s", strucnode->id);
  writeTypeNode(file, (GenericNode*) strucnode->type);

  featvals = strucnode->featvals;

  while (featvals) {
    writeGenericNode(file, featvals->item);
    featvals = featvals->tail;
  }

  fprintf(file, ")");
}

void writeFeatvalNode(FILE* file, GenericNode* node) {
  FeatvalNode* featvalnode = (FeatvalNode*) node;

  fprintf(file, "(V");

  writeFlags(file, node);
  fprintf(file, "%s", featvalnode->id);
  fprintf(file, "\"%s\"", featvalnode->feature);
  writeGenericNode(file, featvalnode->value);

  fprintf(file, ")");
}

void writeListNode(FILE* file, GenericNode* node) {
  ListNode* listnode = (ListNode*) node;
  List* structs;

  fprintf(file, "(L");
  
  writeFlags(file, node);
  fprintf(file, "%s", listnode->id);

  structs = listnode->structs;

  while (structs) {
    writeGenericNode(file, structs->item);
    structs = structs->tail;
  }

  fprintf(file, ")");
}

void writeSetNode(FILE* file, GenericNode* node) {
  SetNode* setnode = (SetNode*) node;
  List* structs;

  fprintf(file, "(M");
  
  writeFlags(file, node);
  fprintf(file, "%s", setnode->id);

  structs = setnode->structs;

  while (structs) {
    writeGenericNode(file, structs->item);
    structs = structs->tail;
  }

  fprintf(file, ")");
}

void writeFunctionNode(FILE* file, GenericNode* node) {
  FunctionNode* functionnode = (FunctionNode*) node;
  List* structs;

  fprintf(file, "(F");

  writeFlags(file, node);
  fprintf(file, "%s", functionnode->id);

  writeTypeNode(file, (GenericNode*) functionnode->type);
  fprintf(file, "\"%s\"", functionnode->functor);

  structs = functionnode->structs;

  while (structs) {
    writeGenericNode(file, structs->item);
    structs = structs->tail;
  }

  fprintf(file, ")");
}

void writeRelationNode(FILE* file, GenericNode* node) {
  RelationNode* relationnode = (RelationNode*) node;
  List* structs;

  fprintf(file, "(D");

  writeFlags(file, node);
  fprintf(file, "%s", relationnode->id);

  fprintf(file, "\"%s\"", relationnode->functor);

  structs = relationnode->structs;

  while (structs) {
    writeGenericNode(file, structs->item);
    structs = structs->tail;
  }

  fprintf(file, ")");
}

void writeRefNode(FILE* file, GenericNode* node) {
  RefNode* refnode = (RefNode*) node;

  fprintf(file, "(#");

  writeFlags(file, node);

  fprintf(file, "%s %d)", refnode->id, refnode->ref);
}

void writeReentNode(FILE* file, GenericNode* node) {
  ReentNode* reentnode = (ReentNode*) node;

  fprintf(file, "(R");

  writeFlags(file, node);
  fprintf(file, "%s %d", reentnode->id, reentnode->ref);

  writeGenericNode(file, reentnode->value);

  fprintf(file, ")");
} 

void writeTreeNode(FILE* file, GenericNode* node) {
  TreeNode* treenode = (TreeNode*) node;
  List* subtrees = treenode->subtrees;

  fprintf(file, "(T");

  writeFlags(file, node);
  fprintf(file, "%s\"%s\"", treenode->id, treenode->label);
  if (treenode->arclabel)
    fprintf(file, "\"%s\"", treenode->arclabel);
  fprintf(file, "%s", treenode->linkid);

  while (subtrees) {
    writeGenericNode(file, subtrees->item);
    subtrees = subtrees->tail;
  }

  fprintf(file, ")");
}

void writeAtomNode(FILE* file, GenericNode* node) {
  AtomNode* atomnode = (AtomNode*) node;
  
  fprintf(file, "(A");

  writeFlags(file, node);

  fprintf(file, "%s\"%s\"", atomnode->id, atomnode->atom);

  fprintf(file, ")");
}

void writeTailNode(FILE* file, GenericNode* node) {
  TailNode* tailnode = (TailNode*) node;

  fprintf(file, "(Z");

  writeFlags(file, node);

  fprintf(file, "%s", tailnode->id);

  writeGenericNode(file, tailnode->tail);
  
  fprintf(file, ")");
}

void writeRestNode(FILE* file, GenericNode* node) {
  RestNode* restnode = (RestNode*) node;

  fprintf(file, "(Y");

  writeFlags(file, node);

  fprintf(file, "%s", restnode->id);

  writeGenericNode(file, restnode->rest);
  
  fprintf(file, ")");
}


void writeDisjunctionNode(FILE* file, GenericNode* node) {
  DisjunctionNode* disjnode = (DisjunctionNode*) node;
  List* operands;

  fprintf(file, "(O");

  writeFlags(file, node);

  fprintf(file, "%s", disjnode->id);

  operands = disjnode->operands;
  while (operands) {
    writeGenericNode(file, operands->item);
    operands = operands->tail;
  }

  fprintf(file, ")");
}


void writeConjunctionNode(FILE* file, GenericNode* node) {
  ConjunctionNode* conjnode = (ConjunctionNode*) node;
  List* operands;

  fprintf(file, "(C");

  writeFlags(file, node);

  fprintf(file, "%s", conjnode->id);

  operands = conjnode->operands;
  while (operands) {
    writeGenericNode(file, operands->item);
    operands = operands->tail;
  }

  fprintf(file, ")");
}

  

void writeGenericNode(FILE* file, GenericNode* node) {
  switch (node->nodetype) {
  case NODETYPE_TYPE_NODE:
    writeTypeNode(file, node);
    break;
  case NODETYPE_STRUC_NODE:
    writeStrucNode(file,  node);
    break;
  case NODETYPE_FEATVAL_NODE:
    writeFeatvalNode(file, node);
    break;
  case NODETYPE_LIST_NODE:
    writeListNode(file, node);
    break;
  case NODETYPE_SET_NODE:
    writeSetNode(file, node);
    break;
  case NODETYPE_FUNCTION_NODE:
    writeFunctionNode(file, node);
    break;
  case NODETYPE_REL_NODE:
    writeRelationNode(file, node);
    break;
  case NODETYPE_REF_NODE:
    writeRefNode(file, node);
    break;
  case NODETYPE_REENT_NODE:
    writeReentNode(file, node);
    break;
  case NODETYPE_TREE_NODE:
    writeTreeNode(file, node);
    break;
  case NODETYPE_ATOM_NODE:
    writeAtomNode(file, node);
    break;
  case NODETYPE_TAIL_NODE:
    writeTailNode(file, node);
    break;
  case NODETYPE_REST_NODE:
    writeRestNode(file, node);
    break;
  case NODETYPE_DISJUNCTION_NODE:
    writeDisjunctionNode(file, node);
    break;
  case NODETYPE_CONJUNCTION_NODE:
    writeConjunctionNode(file, node);
    break;
  }
}

NodeIdMap* addToMap(NodeIdMap* map, GenericNode* node, int id) {
  if (map == NULL) {
    map = new NodeIdMap;
    map->maxID = 0;
  }

  NodeIdMapping* m = new NodeIdMapping;

  m->node = node;
  m->id = new char[20];
  sprintf(m->id, "%d", id);
  m->next = map->mappings;

  map->mappings = m;

  if (map->maxID < id)
    map->maxID = id;

  return map;
}

GenericNode* findNodeById(NodeIdMap* map, char* id) {
  NodeIdMapping* m = map->mappings;

  while (m) {
    if (strcmp(m->id, id) == 0)
      return m->node;
    m = m->next;
  }

  return NULL;
}

void destroyMap(NodeIdMap* map)
{
  NodeIdMapping* mdel;
  NodeIdMapping* m = map->mappings;

  while (m) {
    mdel = m;
    m = m->next;
    destroyNode(mdel->node);
    delete [] mdel->id;
    delete mdel;
  }

  delete map;
}   

void printType(TypeNode* node, int level)
{
  space(level);
  printf("%s TypeNode %s\n", node->id, node->name);
}

void printStruc(StrucNode* node, int level)
{
  space(level);
  printf("%s StrucNode\n", node->id);
  printType(node->type, level + 1);
  catList(node->featvals, level + 1);
}

void printFeatval(FeatvalNode* node, int level)
{
  space(level);
  printf("%s FeatvalNode\n", node->id);
  space(level + 1);
  printf("%s\n", node->feature);
  printNode(node->value, level + 1);
}

void printList(ListNode* node, int level)
{
  space(level);
  printf("%s ListNode\n", node->id);
  catList(node->structs, level + 1);
}

void printSet(SetNode* node, int level)
{
  space(level);
  printf("%s SetNode\n", node->id);
  catList(node->structs, level + 1);
}

void printFunction(FunctionNode* node, int level)
{
  space(level);
  printf("%s FunctionNode\n", node->id);
  printType(node->type, level + 1);
  space(level + 1);
  printf("%s\n", node->functor);
  catList(node->structs, level + 1);
}

void printRelation(RelationNode* node, int level)
{
  space(level);
  printf("%s RelationNode\n", node->id);
  space(level + 1);
  printf("%s\n", node->functor);
  catList(node->structs, level + 1);
}

void printRef(RefNode* node, int level) {
  space(level);
  printf("%s RefNode %d\n", node->id, node->ref);
}

void printReent(ReentNode* node, int level) {
  space(level);
  printf("%s ReentNode %d\n", node->id, node->ref);
  printNode(node->value, level + 1);
}

void printTree(TreeNode* node, int level) {
  space(level);
  printf("%s TreeNode %s %s\n", node->id, node->label, node->linkid);
  catList(node->subtrees, level + 1);
}

void printAtom(AtomNode* node, int level) {
  space(level);
  printf("%s AtomNode %s\n", node->id, node->atom);
}

void printNode(GenericNode* node, int level)
{
  if (node == NULL)
    return;
  switch (node->nodetype) {
  case NODETYPE_TYPE_NODE:
    printType((TypeNode*) node, level);
    break;
  case NODETYPE_STRUC_NODE:
    printStruc((StrucNode*) node, level);
    break;
  case NODETYPE_FEATVAL_NODE:
    printFeatval((FeatvalNode*) node, level);
    break;
  case NODETYPE_LIST_NODE:
    printList((ListNode*) node, level);
    break;
  case NODETYPE_SET_NODE:
    printSet((SetNode*) node, level);
    break;
  case NODETYPE_FUNCTION_NODE:
    printFunction((FunctionNode*) node, level);
    break;
  case NODETYPE_REL_NODE:
    printRelation((RelationNode*) node, level);
    break;
  case NODETYPE_REF_NODE:
    printRef((RefNode*) node, level);
    break;
  case NODETYPE_REENT_NODE:
    printReent((ReentNode*) node, level);
    break;
  case NODETYPE_TREE_NODE:
    printTree((TreeNode*) node, level);
    break;
  case NODETYPE_ATOM_NODE:
    printAtom((AtomNode*) node, level);
    break;
  default:
    printf("Illegal node type %d\n", node->nodetype);
    break;
  }
}


