/*  $Id: scrollcanvas.h,v 1.29 2003/05/22 16:43:14 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @file
    Declarations for the <code>ScrollCanvas</code> class.
    @author Carsten He */


#ifndef __SCROLLCANVAS_H
#define __SCROLLCANVAS_H

#include "canvas.h"


/** This class provides a scrollable container for a <code>Canvas</code>.
    @author Carsten He */
class ScrollCanvas : public QWidget {
  /** The horizontal scrolbar. */
  MyScrollBar* h_scroll;


  /** The vertical scrollbar. */
  MyScrollBar* v_scroll;


  /** The <code>Canvas</code> which is displayed within this scrollable
      container. */
  Canvas* canvas;
  
 public:
  /** Creates a new <code>ScrollCanvas</code>.
      @param options the global options object.
      @param d the document to be displayed.
      @param rootNode the root node in the document
      @param parent the parent widget (must be <code>ViewCanvas</code>). */
  ScrollCanvas(Options* options, Document* d, GenericNode* rootNode, QWidget* parent);


  /** Destroys this <code>ScrollCanvas</code> and the <code>Canvas</code> contained
      in it. */
  ~ScrollCanvas();


  /** Called when the parent window was resized.
      @param ev the resize event. */
  virtual void resizeEvent(QResizeEvent *ev);


  /** Returns the <code>Canvas</code> contained in this <code>ScrollCanvas</code>.
      @return the canvas. */
  Canvas* getCanvas();


  /** Returns the size of this <code>ScrollCanvas</code>.
      @return the size. */
  QSize size() const;
};

#endif
