/*  $Id: vcanvas.cpp,v 1.39 2003/05/24 01:55:02 dm Exp $

    This file is part of the

    GRammar vISUalization tool (GRISU)

    Developed by Holger Wunsch and Detmar Meurers
    Copyright (C) 1998-2003 by the developers
    All rights reserved

    Seminar fr Sprachwissenschaft, Universitt Tbingen and
    Department of Linguistics, The Ohio State University

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @file

    Implementation of the <code>ViewCanvas</code> class.

    @author Holger Wunsch */

#include <qmessagebox.h>
#include <qaccel.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qiconset.h>
#include <kicontheme.h>
#include <kfiledialog.h>
#include <qfiledialog.h>

#include "global.h"
#include "vcanvas.h"
#include "gspacingdialog.h"
#include "vcanvas.moc"
#include "find.h"
#include "NodeAddress.h"
#include "config.h"

#include "glatexrenderer.h"


ViewCanvas::ViewCanvas( Document* d, int what, Options *op,
			QWidget *parent, const char *name ):
  KMainWindow(parent, name) {
  QPixmap pix;
  QImage image;
  QIconSet icon;
  KIconLoader* loader = KApplication::kApplication()->iconLoader();

  QAccel* accel = new QAccel(this);

  showing = what;

  document=d;
  
  options=new Options(*op);

  QString help;
  if (op->showAppName)
    help = d->getName() + "  -  " + APP_NAME + " " + GRISU_VERSION;
  else
    help = d->getName();
  setPlainCaption(help);

  if (what == DRAW_STRUC)
    scrollCanvas   = new ScrollCanvas(options, d, d->data->at(0), this);
  else if (what == DRAW_TREE) {
    if (d->tree)
      scrollCanvas   = new ScrollCanvas(options, d, d->tree, this);
    else
      scrollCanvas   = new ScrollCanvas(options, d, d->data->at(0), this);
  }

  setCentralWidget(scrollCanvas);
  canvas = scrollCanvas->getCanvas();
  
  menuBar = KMainWindow::menuBar();
  
  // File Menu
  fileMenu = new QPopupMenu;
  
  pix = loader->loadIcon("filefloppy", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  fileMenu->insertItem(icon, "&Save...", this,
		       SLOT(saveSlot()), ALT + Key_S);

  fileMenu->insertSeparator();

  pix = loader->loadIcon("fileprint", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  fileMenu->insertItem(icon, "&Print...", this,
		       SLOT( printSlot() ), ALT + Key_P);
  
  /* BETA FEATURE: LaTeX Output of feature structures. Remove the
     comment characters from the next three lines in order to activate
     this. */
  fileMenu->insertSeparator();
  fileMenu->insertItem("Render &LaTeX...", this,
  		       SLOT(renderLatexSlot(void)));

  fileMenu->insertSeparator();

  fileMenu->insertItem ("&Close This Window", this,
			SLOT( quitWindowSlot() ), ALT + Key_W );
  
  menuBar->insertItem("&File", fileMenu);

  accel->connectItem(accel->insertItem(ALT + Key_F4),
		     this, SLOT(quitWindowSlot()));
  
  
  editMenu = new QPopupMenu;
  editMenu->setCheckable(true);
  
  pix = loader->loadIcon("expand", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  editMenu->insertItem (icon, "&Expand Tags", this,
			SLOT( expandReentranciesSlot() ), CTRL + Key_E );
  editMenu->insertItem("&Reset Structure", this,
		       SLOT(resetStructureSlot()), CTRL + Key_R);
  editMenu->insertSeparator();
  pix = loader->loadIcon("magglass", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  editMenu->insertItem(icon, "Find...", this,
		       SLOT(searchSlot()), CTRL + Key_F);

  accel->connectItem(accel->insertItem(Key_F3),
		     this, SLOT(findNextSlot()));
  accel->connectItem(accel->insertItem(CTRL + Key_G),
  		     this, SLOT(findNextSlot()));
  
  menuBar->insertItem("&Edit",editMenu);
  

  dataMenu = new QPopupMenu;
  pix = loader->loadIcon("showtree", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  dataMenu->insertItem (icon, "Show &Tree", this,
			SLOT( showTreeSlot()), ALT + Key_T );
  pix = loader->loadIcon("showstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  dataMenu->insertItem (icon, "Show &Structure", this,
			SLOT( showStructureSlot() ), ALT + Key_S);

  dataMenu->insertSeparator();

  dataMenu->insertItem ("Show Tr&ee In New Window", this,
			SLOT( showTreeNewWindowSlot()));
  dataMenu->insertItem ("Show St&ructure In New Window", this,
			SLOT(showStructureNewWindowSlot()));

  dataMenu->insertSeparator();

  pix = loader->loadIcon("prevstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  dataMenu->insertItem(icon, "&Previous Data", this, 
			    SLOT(previousStructureSlot()), Key_P);
  pix = loader->loadIcon("nextstruc", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  dataMenu->insertItem(icon, "&Next Data", this, 
			    SLOT(nextStructureSlot()), Key_N);

  menuBar->insertItem("&Data", dataMenu);
  

  optionsMenu = new QPopupMenu;
  
  optionsMenu->insertItem("Window &Fits Structure Size", this, SLOT(toggleFitSlot()), 0, MID_VCOPTIONS_FIT);
  optionsMenu->setItemChecked(MID_VCOPTIONS_FIT, options->fitToDrawing);
  optionsMenu->insertItem("&Highlight Structures", this, SLOT(toggleHighlightSlot()), 0, MID_VCOPTIONS_HIGHLIGHT);
  optionsMenu->setItemChecked(MID_VCOPTIONS_HIGHLIGHT, options->highlightStructures);
  optionsMenu->insertItem("&Display Hidden Nodes", this, SLOT(toggleShowHiddenSlot()), 0, MID_VCOPTIONS_SHOWHIDDEN);
  optionsMenu->setItemChecked(MID_VCOPTIONS_SHOWHIDDEN, options->drawHidden);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem("&Structure Spacing...", this,
			  SLOT(optionsSlot()));
  
  menuBar->insertItem("&Options", optionsMenu);

  windowMenu = new QPopupMenu;

  pix = loader->loadIcon("closeviews", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  windowMenu->insertItem(icon, "Close &All Data Windows", parent,
		       SLOT(closeAllSlot()), ALT + Key_A);

  windowMenu->insertSeparator();

  pix = loader->loadIcon("raisewindow", KIcon::User);
  image = pix.convertToImage();
  pix.convertFromImage(image.smoothScale(16, 16));
  icon = QIconSet(pix);
  windowMenu->insertItem(icon, "Raise &Main Window", this,
		       SLOT(raiseMainWindowSlot()), ALT + Key_M);
  
  menuBar->insertItem("&Window", windowMenu);


  QObject::connect(this, SIGNAL(previousStructureSignal(ViewCanvas*)), 
		   parent, SLOT(previousStructureSlot(ViewCanvas*)));
  QObject::connect(this, SIGNAL(nextStructureSignal(ViewCanvas*)), 
		   parent, SLOT(nextStructureSlot(ViewCanvas*)));


  
  // Create toolbar...
  mainToolBar = toolBar();
  
  mainToolBar->setTitle("Grisu Data Toolbar");
  
  pix = loader->loadIcon("filefloppy", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
			    this, SLOT(saveSlot()),
			    TRUE, "Save");

  pix = loader->loadIcon("fileprint", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
			    this, SLOT(printSlot()),
			    TRUE, "Print");

  mainToolBar->insertSeparator();
  mainToolBar->insertSeparator();
  
  pix = loader->loadIcon("expand", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
			    this, SLOT(expandReentranciesSlot()),
			    TRUE, "Expand Tags");
  
  pix = loader->loadIcon("maximize", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()), 
			    this, SLOT(toggleFitSlot()),
			    TRUE, "Window Fits Structure Size ON/OFF");

  mainToolBar->insertSeparator();

  pix = loader->loadIcon("showtree", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    this, SLOT(showTreeSlot()),
			    TRUE, "Show Tree");

  pix = loader->loadIcon("showstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    this, SLOT(showStructureSlot()),
			    TRUE, "Show Structure");
  
  mainToolBar->insertSeparator();

  pix = loader->loadIcon("magglass", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    this, SLOT(searchSlot()),
			    TRUE, "Find");

  mainToolBar->insertSeparator();

  pix = loader->loadIcon("prevstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    this, SLOT(previousStructureSlot()),
			    TRUE, "Previous Data");

  pix = loader->loadIcon("nextstruc", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    this, SLOT(nextStructureSlot()),
			    TRUE, "Next Data");

  mainToolBar->insertSeparator();
  mainToolBar->insertSeparator();

  pix = loader->loadIcon("closewindow", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    this, SLOT(quitWindowSlot()),
			    TRUE, "Close This Window");
  
  mainToolBar->insertSeparator();

  pix = loader->loadIcon("closeviews", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    parent, SLOT(closeAllSlot()),
			    TRUE, "Close All Data Windows");

  mainToolBar->insertSeparator();

  pix = loader->loadIcon("raisewindow", KIcon::User);
  mainToolBar->insertButton(pix, -1, SIGNAL(clicked()),
			    this, SLOT(raiseMainWindowSlot()),
			    TRUE, "Raise Main Window");

  statusBar = KMainWindow::statusBar();

  statusBar->insertItem("", 1, 1);
  statusBar->setItemAlignment(1, Qt::AlignLeft);


  //    KMainWindow::toolBar()->show();
  //resizeEvent(NULL);
  
  connect( canvas, SIGNAL(redrawSignal()),
	   this, SLOT(redrawSlot()) );
  
  connect( this, SIGNAL(redrawSignal()),
	   this, SLOT(redrawSlot()) );
  
  setMaximumSize(application->desktop()->width(), 
		 application->desktop()->height());

  finder = NULL;
  
  findDlg = NULL;

  if (globalOptions->expandReentrancies)
    expandReentranciesSlot();
  else
    redrawSlot();

}


ViewCanvas::~ViewCanvas() {
  if (findDlg != NULL) {
    findDlg->close();
  }
  if (finder != NULL)
    delete finder;

  delete options;
}


void ViewCanvas::resizeEvent(QResizeEvent *ev) {
  KMainWindow::resizeEvent(ev);
  
  scrollCanvas->resizeEvent(ev);
}


void ViewCanvas::redrawSlot() {
  canvas->repaint(true);

  //  repaint();
  if (options->fitToDrawing) {
    QSize maxSize;
#if (QT_VERSION >= 0x030000)
#if (QT_VERSION >= 0x030101)
    QRect geom = KApplication::kApplication()->desktop()->availableGeometry();
#else
    QRect geom = KApplication::kApplication()->desktop()->screenGeometry();
#endif
    maxSize.setWidth(geom.width());
    maxSize.setHeight(geom.height());
#else
    maxSize.setWidth(maximumSize().width());
    maxSize.setHeight(maximumSize().height());
#endif

    // resize ... 
    int maw = canvas->getMaxSize().width() + 30;
    int mah = canvas->getMaxSize().height() + 30;
    
    
    if (mah < 50) mah=100;
    
    int six = maw + (width() - canvas->width()) ;
    int siy = mah + (height() - canvas->height()) ;
    
    if (six > maxSize.width()) 
      six = maxSize.width();
    if (siy > maxSize.height())
      siy = maxSize.height();
    
    resize(six,siy);

    QRect geo = geometry();
    QRect fgeo = frameGeometry();
    
    int fdx = fgeo.width() - geo.width();
    int fdy = fgeo.height() - geo.height();
    
    //move ..
    int x = pos().x();
    int y = pos().y();


    if (x + six > maxSize.width() || y + siy > maxSize.height()) {
      if (maxSize.width() - six < x) 
	x = maxSize.width() - six - fdx;
      
      if (maxSize.height() - siy < y) 
	y = maxSize.height() - siy - fdy; 
      
      if (x < 0)
	x = 0;

      if (y < 0)
	y = 0;

      move(x,y);
    }
  }
}


void ViewCanvas::toggleShowHiddenSlot() {
  options->drawHidden = !(options->drawHidden);
  optionsMenu->setItemChecked(MID_VCOPTIONS_SHOWHIDDEN, options->drawHidden);
  
  emit redrawSignal();
}


void ViewCanvas::redraw() {
  emit redrawSignal();
}


void ViewCanvas::toggleFitSlot() {
  options->fitToDrawing = !(options->fitToDrawing);
  optionsMenu->setItemChecked(MID_VCOPTIONS_FIT, options->fitToDrawing);
  if (!options->fitToDrawing) {
    resize(480, 480);
  }
  emit redrawSignal();
}


void ViewCanvas::toggleHighlightSlot() {
  options->highlightStructures = !(options->highlightStructures);
  optionsMenu->setItemChecked(MID_VCOPTIONS_HIGHLIGHT, options->highlightStructures);
  emit redrawSignal();
}


void ViewCanvas::expandReentranciesSlot() {
  GenericNode* node;
  if (showing == DRAW_STRUC)
    node = document->data->at(0);
  else if (showing == DRAW_TREE)
    node = (GenericNode*) document->tree;
    
  QList<int>* expRef = new QList<int>();

  NodeAddress na1;
  NodeAddress na2(na1, node);
  expandReentrancies(node, na2, expRef);

  delete expRef;

  emit redrawSignal();
}


void ViewCanvas::expandReentrancies(GenericNode* node, NodeAddress addr, QList<int>* expRef) {
  DisplayableNode* nd = canvas->getDispNodes()->findDisplayableNode(addr);
  if (nd) {
    if (nd->flags & NODEFLAG_HIDDEN)
      return;
  }
  else {
    if (node->flags & NODEFLAG_HIDDEN)
      return;
  }

  switch (node->nodetype) {
  case NODETYPE_TREE_NODE:
    expandReentranciesTreeNode((TreeNode*) node, addr, expRef);
    break;
  case NODETYPE_STRUC_NODE:
    expandReentranciesStrucNode((StrucNode*) node, addr, expRef);
    break;
  case NODETYPE_FEATVAL_NODE:
    expandReentranciesFeatvalNode((FeatvalNode*) node, addr, expRef);
    break;
  case NODETYPE_LIST_NODE:
    expandReentranciesListNode((ListNode*) node, addr, expRef);
    break;
  case NODETYPE_TAIL_NODE:
    expandReentranciesTailNode((TailNode*) node, addr, expRef);
    break;
  case NODETYPE_SET_NODE:
    expandReentranciesSetNode((SetNode*) node, addr, expRef);
    break;
  case NODETYPE_REST_NODE:
    expandReentranciesRestNode((RestNode*) node, addr, expRef);
    break;
  case NODETYPE_REF_NODE:
    expandReentranciesRefNode((RefNode*) node, addr, expRef);
    break;
  case NODETYPE_FUNCTION_NODE:
    expandReentranciesFunctionNode((FunctionNode*) node, addr, expRef);
    break;
  case NODETYPE_REL_NODE:
    expandReentranciesRelationNode((RelationNode*) node, addr, expRef);
    break;
  case NODETYPE_ATOM_NODE:
    break;
  case NODETYPE_DISJUNCTION_NODE:
    expandReentranciesDisjunctionNode((DisjunctionNode*) node, addr, expRef);
    break;
  case NODETYPE_CONJUNCTION_NODE:
    expandReentranciesDisjunctionNode((DisjunctionNode*) node, addr, expRef);
    break;
  default:
    printf("Illegal node type %d in expandReentrancies\n", node->nodetype);
    break;
  }
}


void ViewCanvas::expandReentranciesTreeNode(TreeNode* node, NodeAddress addr, QList<int>* expRef) {
  GenericNode* struc = canvas->getDocument()->findNodeById(node->linkid);
  
  if (struc) {
    NodeAddress addrKid(addr, struc);
    expandReentrancies(struc, addrKid, expRef);
  }

  List* subtrees = node->subtrees;
  while (subtrees) {
    NodeAddress addrKid(addr, subtrees->item);
    expandReentrancies(subtrees->item, addrKid, expRef);
    subtrees = subtrees->tail;
  }
}

    
void ViewCanvas::expandReentranciesStrucNode(StrucNode* node, NodeAddress addr, QList<int>* expRef) {
  List* featvals = node->featvals;
  
  while (featvals) {
    NodeAddress addrKid(addr, (featvals->item));
    expandReentrancies(featvals->item, addrKid, expRef);
    featvals = featvals->tail;
  }
}


void ViewCanvas::expandReentranciesFeatvalNode(FeatvalNode* node, NodeAddress addr, QList<int>* expRef) {
  NodeAddress addrKid(addr, node->value);
  expandReentrancies(node->value, addrKid, expRef);
}


void ViewCanvas::expandReentranciesListNode(ListNode* node, NodeAddress addr, QList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }

  if (node->tail)
    // Tail nodes themselves are not drawn -> their ID is not included in node address!!
    expandReentrancies(node->tail, addr, expRef);
}


void ViewCanvas::expandReentranciesTailNode(TailNode* node, NodeAddress addr, QList<int>* expRef) {
  NodeAddress addrKid(addr, node->tail);
  expandReentrancies(node->tail, addrKid, expRef);
}


void ViewCanvas::expandReentranciesSetNode(SetNode* node, NodeAddress addr, QList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }

  if (node->rest)
    // Rest nodes themselves are not drawn -> their ID is not included in node address!!
    expandReentrancies(node->rest, addr, expRef);
}


void ViewCanvas::expandReentranciesRestNode(RestNode* node, NodeAddress addr, QList<int>* expRef) {
  NodeAddress addrKid(addr, node->rest);
  expandReentrancies(node->rest, addrKid, expRef);
}


void ViewCanvas::expandReentranciesRefNode(RefNode* node, NodeAddress addr, QList<int>* expRef) {
  int* ref = expRef->first();

  while (ref) {
    if (*ref == node->ref)
      return;
    ref = expRef->next();
  }

  DisplayableNode* nd = canvas->getDispNodes()->addDisplayableNode(addr, (GenericNode*) node);
  
  if (nd) {
    expRef->append(&node->ref);
    nd->flags |= NODEFLAG_EXPANDED;
    ReentNode* reent;
    reent = document->findReentrancy(node->ref);
    if (reent != NULL) {
      NodeAddress addrKid(addr, reent->value);
      expandReentrancies(reent->value, addrKid, expRef);
    }
    else {
      printf("ViewCanvas::expandReentranciesRefNode\n");
      printf("INTERNAL ERROR: ReentrNode with number %ld not in data\n", node->ref);
    }
  }
  else {
    printf("ViewCanvas::expandReentranciesRefNode\n");
    printf("INTERNAL ERROR: Node couldn't be added to the list of displayable nodes.\n");
  }
} 


void ViewCanvas::expandReentranciesFunctionNode(FunctionNode* node, NodeAddress addr, QList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }
}


void ViewCanvas::expandReentranciesRelationNode(RelationNode* node, NodeAddress addr, QList<int>* expRef) {
  List* structs = node->structs;

  while (structs) {
    NodeAddress addrKid(addr, structs->item);
    expandReentrancies(structs->item, addrKid, expRef);
    structs = structs->tail;
  }
}


void ViewCanvas::expandReentranciesDisjunctionNode(DisjunctionNode* node, NodeAddress addr, QList<int>* expRef) {
  List* operands = node->operands;

  while (operands) {
    NodeAddress addrChild(addr, operands->item);
    expandReentrancies(operands->item, addrChild, expRef);
    operands = operands->tail;
  }
}


void ViewCanvas::expandReentranciesConjunctionNode(ConjunctionNode* node, NodeAddress addr, QList<int>* expRef) {
  List* operands = node->operands;

  while (operands) {
    NodeAddress addrChild(addr, operands->item);
    expandReentrancies(operands->item, addrChild, expRef);
    operands = operands->tail;
  }
}


void ViewCanvas::saveStructure(void) {
  QString Filename;
  bool ok = false;

  while (!ok) {
    if (options->useKFileDialog) 
      Filename = KFileDialog::getSaveFileName(NULL, 
					      "*.grisu|Grisu data files (*.grisu)\n*|All files (*)", 
					      this, "Save Data");
    else
      Filename = QFileDialog::getSaveFileName(NULL, 
					      "Grisu data files (*.grisu);;All files (*)", 
					      this, NULL, "Save Data");
    if (Filename.length() > 0) {
      if (QFile::exists(Filename)) {
	char buf[500];
	sprintf(buf, "The file %s already exists.\nOverwrite?", (const char*) Filename.local8Bit());
	switch (QMessageBox::information(this, "Save", buf, "&Yes", "&No", "Cancel", 0, 2)) {
	case 0:
	  ok = true;
	  break;
	case 1:
	  break;
	case 2:
	  return;
	}
      }
      else
	ok = true;
    }
    else if (Filename.isEmpty())
      return;
  }
  
  FILE* file = fopen(Filename.local8Bit(), "w");
  
  if (file) {
    setCursor(waitCursor);
    
    document->write(file);
    fprintf(file, "\n");
    fclose(file);
    
    setCursor(arrowCursor);
  }
  else {
    char buf[500];
    sprintf(buf, "Could not open file %s", (const char*) Filename.local8Bit());
    QMessageBox::critical(this, "Save", buf);
  }
}
  

void ViewCanvas::saveSlot(void) {
  saveStructure();
}


void ViewCanvas::quitWindowSlot() {
  close();
}


void ViewCanvas::hideUnhidePopupSlot() {
  canvas->hideUnhide(canvas->getDispNodes()->findDisplayableNode(canvas->getKlickedOn()->addr));
}


void ViewCanvas::expandLocalPopupSlot() {
  NodeAddress addr = canvas->getKlickedOn()->addr;
  QList<int>* expRef = new QList<int>();

  expandReentrancies(canvas->getDispNodes()->findDisplayableNode(addr)->node, addr, expRef);

  delete expRef;

  emit redrawSignal();
}


void ViewCanvas::printSlot() {
  QPrinter printer;
  options->readPrinterOptions(printer);
  if(printer.setup(this)) {
    options->writePrinterOptions(printer);
    canvas -> printToPrinter(printer);
  }
}


void ViewCanvas::searchSlot() {
  if (findDlg == NULL) {
    QString dlgCaption("Find in ");
    dlgCaption.append(document->getName());
    
    findDlg = new StrucFindDialog(dlgCaption, this);
    connect(findDlg, SIGNAL(findSignal()), this, SLOT(findSlot()));
    connect(findDlg, SIGNAL(findNextSignal()), this, SLOT(findNextSlot()));
    connect(findDlg, SIGNAL(cancelSignal()), this, SLOT(cancelSearchSlot()));
  }
  findDlg->show();
}


void ViewCanvas::contextMenuSearch(char* searchTerm, const NodeAddress addr) {
  if (findDlg != NULL)
    findDlg->close();

  if (finder != NULL)
    delete finder;

  QString dlgCaption("Find in ");
  dlgCaption.append(document->getName());

  findDlg = new StrucFindDialog(dlgCaption, searchTerm, this);
  connect(findDlg, SIGNAL(findSignal()), this, SLOT(findSlot()));
  connect(findDlg, SIGNAL(findNextSignal()), this, SLOT(findNextSlot()));
  connect(findDlg, SIGNAL(cancelSignal()), this, SLOT(cancelSearchSlot()));
  findDlg->show();

  QArray<int>* types = new QArray<int>(15);
  (*types)[0] = NODETYPE_TYPE_NODE;
  (*types)[1] = NODETYPE_STRUC_NODE;
  (*types)[2] = NODETYPE_FEATVAL_NODE;
  (*types)[3] = NODETYPE_LIST_NODE;
  (*types)[4] = NODETYPE_SET_NODE;
  (*types)[5] = NODETYPE_FUNCTION_NODE;
  (*types)[6] = NODETYPE_REF_NODE;
  (*types)[7] = NODETYPE_REENT_NODE;
  (*types)[8] = NODETYPE_TREE_NODE;
  (*types)[9] = NODETYPE_ATOM_NODE;
  (*types)[10] = NODETYPE_REL_NODE;
  (*types)[11] = NODETYPE_TAIL_NODE;
  (*types)[12] = NODETYPE_REST_NODE;
  (*types)[13] = NODETYPE_DISJUNCTION_NODE;
  (*types)[14] = NODETYPE_CONJUNCTION_NODE;

  finder = new Finder(document, types, findDlg->text().local8Bit(), findDlg->ignoreCaseChecked());

  if (showing == DRAW_STRUC)
    finder->findGenericNode(document->data->at(0), NodeAddress());
  else
    finder->findGenericNode((GenericNode*) document->tree, NodeAddress());
  if (document->rel->count() > 0)
    finder->findGenericNode(document->rel->at(0), NodeAddress());

  const NodeAddress* found = finder->first();

  if (found == NULL) {
    QMessageBox::information(findDlg, "Find structure", "No matches found.", QMessageBox::Ok);
    findDlg->close();
    findDlg = NULL;
    delete finder;
    finder = NULL;

    return;
  }

  const NodeAddress* start = found;

  while (true) {
    if (*found == addr)
      break;

    found = finder->next();

    if (found == NULL)
      found = finder->first();

    if (found == start) {
      QMessageBox::information(findDlg, "Find structure", "No matches found.", QMessageBox::Ok);
      findDlg->close();
      findDlg = NULL;
      delete finder;
      finder = NULL;
      
      return;
    }
  }

  found = finder->next();
  if (found == NULL)
    found = finder->first();

  canvas->ensureVisible(found);
  canvas->markFound(found);
}
  

void ViewCanvas::findSlot() {
  QArray<int>* types = new QArray<int>(15);
  (*types)[0] = NODETYPE_TYPE_NODE;
  (*types)[1] = NODETYPE_STRUC_NODE;
  (*types)[2] = NODETYPE_FEATVAL_NODE;
  (*types)[3] = NODETYPE_LIST_NODE;
  (*types)[4] = NODETYPE_SET_NODE;
  (*types)[5] = NODETYPE_FUNCTION_NODE;
  (*types)[6] = NODETYPE_REF_NODE;
  (*types)[7] = NODETYPE_REENT_NODE;
  (*types)[8] = NODETYPE_TREE_NODE;
  (*types)[9] = NODETYPE_ATOM_NODE;
  (*types)[10] = NODETYPE_REL_NODE;
  (*types)[11] = NODETYPE_TAIL_NODE;
  (*types)[12] = NODETYPE_REST_NODE;
  (*types)[13] = NODETYPE_DISJUNCTION_NODE;
  (*types)[14] = NODETYPE_CONJUNCTION_NODE;

  if (finder != NULL)
    delete finder;

  finder = new Finder(document, types, findDlg->text().local8Bit(), findDlg->ignoreCaseChecked());

  if (showing == DRAW_STRUC)
    finder->findGenericNode(document->data->at(0), NodeAddress());
  else
    finder->findGenericNode((GenericNode*) document->tree, NodeAddress());
  if (document->rel->count() > 0)
    finder->findGenericNode(document->rel->at(0), NodeAddress());


  const NodeAddress* found = finder->first();

  if (found != NULL) {
    canvas->ensureVisible(found);
    canvas->markFound(found);
  }
  else {
    QMessageBox::information(findDlg, "Find structure", "No matches found.", QMessageBox::Ok);
    findDlg->close();
    findDlg = NULL;
    delete finder;
    finder = NULL;
  }
}


void ViewCanvas::findNextSlot() {
  if (findDlg == NULL) {
    searchSlot();
    return;
  }

  if (finder == NULL) {
    findSlot();
    return;
  }
    
  const NodeAddress* found = finder->next();
  if (found == NULL) {
    QMessageBox::information(findDlg, "Find", 
			     "The end of the structure has been reached. The search will start again from the top.", 
			     QMessageBox::Ok);
    found = finder->first();
  }
  canvas->ensureVisible(found);
  canvas->markFound(found);
}


void ViewCanvas::cancelSearchSlot() {
  findDlg = NULL;
  delete finder;
  finder = NULL;
  canvas->markFound(NULL);
}


void ViewCanvas::resetStructureSlot() {
  canvas->reset();
}


void ViewCanvas::showTreeSlot() {
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  if (findDlg != NULL) {
    findDlg->close();
    findDlg = NULL;
  }
  showing = DRAW_TREE;
  canvas->setRootNode((GenericNode*) document->tree);
}


void ViewCanvas::showStructureSlot() {
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  if (findDlg != NULL) { 
    findDlg->close();
    findDlg = NULL;
  }
  showing = DRAW_STRUC;
  canvas->setRootNode(document->data->at(0));
}


void ViewCanvas::showTreeNewWindowSlot() {
  mainw->newViewWindow(document, DRAW_TREE);
}


void ViewCanvas::showStructureNewWindowSlot() {
  mainw->newViewWindow(document, DRAW_STRUC);
}


void ViewCanvas::closeEvent(QCloseEvent* ce) {
  emit closing(this);
  KMainWindow::closeEvent(ce);
}


bool ViewCanvas::showsDoc(Document* doc) {
  return doc == document;
}


void ViewCanvas::optionsSlot() {
  GSpacingDialog* dlg = new GSpacingDialog(options, this, NULL, true);
  if (dlg->exec() == QDialog::Accepted) {
    emit redrawSignal();
  }
}


void ViewCanvas::raiseMainWindowSlot(void) {
  parentWidget()->setActiveWindow();
  parentWidget()->raise();
}


void ViewCanvas::previousStructureSlot(void) {
  if (findDlg != NULL)
    findDlg->close();
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  emit previousStructureSignal(this);
}


void ViewCanvas::nextStructureSlot(void) {
  if (findDlg != NULL)
    findDlg->close();
  if (finder != NULL) {
    delete finder;
    finder = NULL;
  }
  emit nextStructureSignal(this);
}


/** Changes the status bar message.
    @param message the new message. */
void ViewCanvas::setStatusBarMessage(const char* message) {
  statusBar->changeItem(QString::fromLocal8Bit(message), 1);
}


/** Writes the currently selected data package to a LaTeX file. */
void ViewCanvas::renderLatexSlot(void) {
  QString Filename;
  bool ok = false;

  while (!ok) {
    if (options->useKFileDialog)
      Filename = KFileDialog::getSaveFileName(NULL, "*.tex|LaTeX files (*.tex)\n*|All files (*)", this, "Render LaTeX");
    else
      Filename = QFileDialog::getSaveFileName(NULL, "LaTeX files (*.tex);;All files (*)", this, NULL, "Render LaTeX");
    if (Filename.length() > 0) {
      if (QFile::exists(Filename)) {
	char buf[500];
	sprintf(buf, "The file %s already exists.\nOverwrite?", (const char*) Filename.local8Bit());
	switch (QMessageBox::information(this, "Render LaTeX", buf, "&Yes", "&No", "Cancel", 0, 2)) {
	case 0:
	  ok = true;
	  break;
	case 1:
	  break;
	case 2:
	  return;
	}
      }
      else
	ok = true;
    }
    else if (Filename.isEmpty())
      return;
  }

  
  GLatexRenderer latex(document, showing, canvas->getDispNodes(), Filename);
  latex.render();
}
 
